import logging
from flask_jwt_extended import jwt_required
from flask_restplus import Namespace, Resource

from portal.rest import rest_helpers
from portal.rest.schemas import permissions_schema, role_schema, roles_schema, \
    user_list_update_schema, users_schema
from portal.service import auth, general
from portal.service.general import find_client_owner
from portal.service.errors import ForbiddenError
from portal.service.permissions import check_client, require_client
from portal.service.roles import add_single_user_to_role, create_role, delete_role, list_roles, \
    remove_single_user_from_role, update_role, update_role_permissions, \
    update_users_membership_in_role

roles_namespace = Namespace('')

log = logging.getLogger(__name__)


@roles_namespace.route('/courses/<string:cid>/roles')
@roles_namespace.param('cid', 'Course id')
class RoleList(Resource):

    @jwt_required
    # @roles_namespace.response(200, 'Roles list', model=roles_schema)
    def get(self, cid):
        client = auth.find_client()
        course = general.find_course(cid)
        return roles_schema.dump(list_roles(course, client))

    @jwt_required
    # @roles_namespace.response(201, 'Role created', model=role_schema)
    def post(self, cid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        perm = ['update_course']
        require_client(client=client, course=course, permissions=perm)

        data = rest_helpers.parse_request_data(
            role_schema, action='create', resource='role'
        )
        new_role = create_role(course, **data)
        return role_schema.dump(new_role)[0], 201


@roles_namespace.route('/courses/<string:cid>/roles/<string:rid>')
@roles_namespace.param('cid', 'Course id')
@roles_namespace.param('rid', 'Role id')
@roles_namespace.response(404, 'Course not found')
@roles_namespace.response(404, 'Role not found')
class RoleResource(Resource):
    @jwt_required
    # @roles_namespace.response(200, 'Role in the course', model=role_schema)
    def get(self, cid: str, rid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        perm = ['view_course_full', 'view_course_limited']
        require_client(client=client, course=course, permissions=perm)

        role = general.find_role(course, rid)
        return role_schema.dump(role)

    @jwt_required
    @roles_namespace.response(204, 'Deleted role')
    def delete(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        perm = ['update_course']
        require_client(client=client, course=course, permissions=perm)

        role = general.find_role(course, rid)
        delete_role(role)
        return '', 204

    @jwt_required
    @roles_namespace.response(204, 'Role updated')
    def put(self, cid: str, rid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        perm = ['write_roles', 'update_course']
        require_client(client=client, course=course, permissions=perm)

        data = rest_helpers.parse_request_data(
            role_schema, action='update', resource='role', partial=True
        )

        role = general.find_role(course, rid)
        update_role(role, data)
        return '', 204


@roles_namespace.route('/courses/<string:cid>/roles/<string:rid>/permissions')
@roles_namespace.param('cid', 'Course id')
@roles_namespace.param('rid', 'Role id')
@roles_namespace.response(404, 'Course not found')
@roles_namespace.response(404, 'Role not found')
class RolePermissions(Resource):

    @jwt_required
    # @roles_namespace.response(200, 'Get permissions for role', model=permissions_schema)
    def get(self, cid: str, rid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)

        # authorization
        if not check_client(client=client, course=course, permissions=['view_course_full']) or \
                (check_client(client=client, course=course, permissions=['view_course_limited'])
                 and find_client_owner(client) in role.clients):
            raise ForbiddenError(uid=client.id)

        return permissions_schema.dump(role.permissions)

    @jwt_required
    # @roles_namespace.response(204, 'Update permissions for course', model=permissions_schema)
    def put(self, cid: str, rid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        perm = ['write_roles', 'update_course']
        require_client(client=client, course=course, permissions=perm)

        data = rest_helpers.parse_request_data(
            permissions_schema, action='update', resource='role_permissions'
        )

        role = general.find_role(course, rid)
        update_role_permissions(role, data)
        return permissions_schema.dump(role.permissions), 204


@roles_namespace.route('/courses/<string:cid>/roles/<string:rid>/users')
@roles_namespace.param('cid', 'Course id')
@roles_namespace.param('rid', 'Role id')
@roles_namespace.response(404, 'Course not found')
@roles_namespace.response(404, 'Role not found')
class RoleUsersList(Resource):
    @jwt_required
    # @roles_namespace.response(200, 'Gets users in the role', model=users_schema)
    def get(self, cid: str, rid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        # authorization

        if not check_client(client=client, course=course, permissions=['view_course_full']) or \
                (check_client(client=client, course=course,
                              permissions=['view_course_limited']) and find_client_owner(client) in role.clients):
            raise ForbiddenError(uid=client.id)

        return users_schema.dump(role.clients)

    @jwt_required
    @roles_namespace.response(204, 'Updates users membership in role')
    def put(self, cid: str, rid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        perm = ['write_roles', 'update_course']
        require_client(client=client, course=course, permissions=perm)

        data = rest_helpers.parse_request_data(
            user_list_update_schema, action='update', resource='role_membership'
        )

        # everything from users_add is added, THEN everything from users_remove
        # is subtracted
        role = general.find_role(course, rid)
        update_users_membership_in_role(role, data)
        return '', 204


@roles_namespace.route(
    '/courses/<string:cid>/roles/<string:rid>/users/<string:uid>')
@roles_namespace.param('cid', 'Course id')
@roles_namespace.param('rid', 'Role id')
@roles_namespace.param('uid', 'User id')
@roles_namespace.response(404, 'Course not found')
@roles_namespace.response(404, 'Role not found')
@roles_namespace.response(404, 'User not found')
class RoleUser(Resource):

    @jwt_required
    @roles_namespace.response(204, 'Adds user to role')
    def put(self, cid: str, rid: str, uid: str):
        client = auth.find_client()
        course = general.find_course(cid)

        # authorization
        perm = ['write_roles', 'update_course']
        require_client(client=client, course=course, permissions=perm)

        role = general.find_role(course, rid)
        user = general.find_user(uid)
        add_single_user_to_role(role, user)
        return '', 204

    @jwt_required
    @roles_namespace.response(204, 'Removes user from role')
    def delete(self, cid: str, rid: str, uid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        perm = ['write_roles', 'update_course']
        require_client(client=client, course=course, permissions=perm)

        role = general.find_role(course, rid)
        user = general.find_user(uid)
        remove_single_user_from_role(role, user)
        return '', 204
